@@ -1,5 +1,5 @@
 from abc import ABC
-from typing import Union
+from typing import Union, List
 
 import numpy as np
 import pandas as pd
@@ -10,12 +10,38 @@ class TimeSeries(ABC):
         self.time: np.ndarray = np.array(time)
 
     def to_df(self) -> pd.DataFrame:
-        return pd.DataFrame(self.__dict__)
+        d = self.__dict__
+        return pd.DataFrame(dict([(k, pd.Series(v)) for k, v in d.items()])).set_index("time")
+
+    def get_duration(self):
+        if not np.array_equal(self.time, np.array(None)) and len(self.time) > 0:
+            if type(self.time[0]) == np.int64:
+                duration = (self.time[-1] - self.time[0]) * 1e-9
+            else:
+                duration = (self.time[-1] - self.time[0])
+        else:
+            duration = 0
+
+        return duration
+
+    def get_sample_rate(self):
+        if not np.array_equal(self.time, np.array(None)) and self.get_duration() > 0:
+            sample_rate = len(self.time) / self.get_duration()
+        else:
+            sample_rate = 0.0
+
+        return sample_rate
+
+    def __len__(self):
+        if np.array_equal(self.time, np.array(None)):
+            return 0
+        else:
+            return len(self.time)
 
     def __repr__(self):
-        duration = (self.time[-1] - self.time[0]) * 1e-9 if not np.array_equal(self.time, np.array(None)) and len(self.time) > 0 else 0
-        sample_rate = len(self.time) / duration if not np.array_equal(self.time, np.array(None)) and duration > 0 else 0.0
-        return "Length: %d, Duration: %.3f s, Mean Samplerate: %.3f Hz" % (len(self.time), duration, sample_rate)
+        return "Length: %d, duration: %.3f s, Mean sample rate: %.3f Hz" % (len(self.time),
+                                                                            self.get_duration(),
+                                                                            self.get_sample_rate())
 
 
 class AccelerationSeries(TimeSeries):
@@ -112,6 +138,13 @@ class GPSSeries(TimeSeries):
         self.speed_acc: np.ndarray = np.array(speed_acc)
         self.utc_time: np.ndarray = np.array(utc_time)
 
+    def to_ipyleaflef(self) -> List[list]:
+        """
+
+        :return: Returns the lat/lon coordinates as list of list for easy visualization using ipyleaflet
+        """
+        return [[lat, lon] for lat, lon in zip(self.lat, self.lon)]
+
 
 class PressureSeries(TimeSeries):
     def __init__(self, time: Union[list, np.ndarray] = None,
